/*
 *  TOPPERS/FMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Flexible MultiProcessor Kernel
 *
 *  Copyright (C) 2006-2009 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 *
 *  嵭Ԥϡʲ(1)(4)ξ˸¤ꡤܥեȥ
 *  ܥեȥѤΤޤࡥʲƱˤѡʣ
 *  ѡۡʰʲѤȸƤ֡ˤ뤳Ȥ̵ǵ롥
 *  (1) ܥեȥ򥽡ɤηѤˤϡ嵭
 *      ɽѾ浪Ӳ̵ݾڵ꤬Τޤޤηǥ
 *      ˴ޤޤƤ뤳ȡ
 *  (2) ܥեȥ򡤥饤֥ʤɡ¾Υեȥȯ˻
 *      ѤǤǺۤˤϡۤȼɥȡ
 *      ԥޥ˥奢ʤɡˤˡ嵭ɽѾ浪Ӳ
 *      ̵ݾڵǺܤ뤳ȡ
 *  (3) ܥեȥ򡤵Ȥ߹ʤɡ¾Υեȥȯ˻
 *      ѤǤʤǺۤˤϡΤ줫ξ
 *      ȡ
 *    (a) ۤȼɥȡѼԥޥ˥奢ʤɡˤˡ嵭
 *        ɽѾ浪Ӳ̵ݾڵǺܤ뤳ȡ
 *    (b) ۤη֤̤ˡˤäơTOPPERSץȤ
 *        𤹤뤳ȡ
 *  (4) ܥեȥѤˤľŪޤϴŪ뤤ʤ»
 *      ⡤嵭ԤTOPPERSץȤդ뤳ȡ
 *      ޤܥեȥΥ桼ޤϥɥ桼Τʤ
 *      ͳ˴Ťᤫ⡤嵭ԤTOPPERSץȤ
 *      դ뤳ȡ
 *
 *  ܥեȥϡ̵ݾڤ󶡤ƤΤǤ롥嵭Ԥ
 *  TOPPERSץȤϡܥեȥ˴ؤơλŪ
 *  ФŬޤơʤݾڤԤʤޤܥեȥ
 *  ѤˤľŪޤϴŪʤ»˴ؤƤ⡤
 *  Ǥʤ
 *
 */

/*
 *  This file contains a driver for the global timers.
 */
#include "kernel_impl.h"
#include "time_event.h"
#include <sil.h>
#include "target_timer.h"

/*
 *  Clear the timer interrupt request
 */
Inline void
target_timer_int_clear()
{
	/* clear pending bit */
	sil_wrw_mem((void *)MPCORE_GTM_ISR, MPCORE_GTM_ISR_EF);
}

/*
 *  Timer initialization
 */
void
target_timer_initialize(intptr_t exinf)
{
	uint32_t cyc = TO_CLOCK(TIC_NUME, TIC_DENO);

	/* Stop the timer */
	if (x_sense_mprc()) {
		sil_wrw_mem((void *)MPCORE_GTM_CNT, 0x000);
	}

	/* Clear pending interrupts */
	target_timer_int_clear();

	/* Set the counter value (1ms per tick) */
	if (x_sense_mprc()) {
		sil_wrw_mem((void *)MPCORE_GTM_COUNT_L, 0);
		sil_wrw_mem((void *)MPCORE_GTM_COUNT_U, 0);
	}

	/* ڥХ塼쥸ν */
	sil_wrw_mem((void *)MPCORE_GTM_COMPV_L, cyc - 1);
	sil_wrw_mem((void *)MPCORE_GTM_COMPV_U, 0x00);

	/* ȥ󥯥 */
	sil_wrw_mem((void *)MPCORE_GTM_AIR, cyc - 1);

	/*
	 * Set the scaler for 1ms per tick and start the timer
	 */
	sil_wrw_mem((void *)MPCORE_GTM_CNT,
				(MPCORE_GTM_PS_1MS << MPCORE_GTM_CNT_PS_OFFSET) |
				MPCORE_GTM_CNT_AI | MPCORE_GTM_CNT_IEN | MPCORE_GTM_CNT_CE | MPCORE_GTM_CNT_EN);

}

/*
 *  Timer termination
 */
void
target_timer_terminate(intptr_t exinf)
{
	/* Clear interrupts */
	target_timer_int_clear();

	if (x_sense_mprc()) {
		/* Stop the timer */
		sil_wrw_mem((void *)MPCORE_GTM_CNT, 0x00);
	}
}

/*
 *  Timer interrupt handler
 */
void
target_timer_handler(void)
{
	ID prcid;

	/* Clear interrupts */
	target_timer_int_clear();

	/* Signal the system tick */
	iget_pid(&prcid);
	i_begin_int((0x10000 << (prcid - 1))|DIC_IRQNO_GTM);
	signal_time();
	i_end_int((0x10000 << (prcid - 1))|DIC_IRQNO_GTM);
}
