/*
 *  TOPPERS/FMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Flexible MultiProcessor Kernel
 *
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 *  Copyright (C) 2005-2013 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 *
 *  嵭Ԥϡʲ(1)(4)ξ˸¤ꡤܥեȥ
 *  ܥեȥѤΤޤࡥʲƱˤѡʣ
 *  ѡۡʰʲѤȸƤ֡ˤ뤳Ȥ̵ǵ롥
 *  (1) ܥեȥ򥽡ɤηѤˤϡ嵭
 *      ɽѾ浪Ӳ̵ݾڵ꤬Τޤޤηǥ
 *      ˴ޤޤƤ뤳ȡ
 *  (2) ܥեȥ򡤥饤֥ʤɡ¾Υեȥȯ˻
 *      ѤǤǺۤˤϡۤȼɥȡ
 *      ԥޥ˥奢ʤɡˤˡ嵭ɽѾ浪Ӳ
 *      ̵ݾڵǺܤ뤳ȡ
 *  (3) ܥեȥ򡤵Ȥ߹ʤɡ¾Υեȥȯ˻
 *      ѤǤʤǺۤˤϡΤ줫ξ
 *      ȡ
 *    (a) ۤȼɥȡѼԥޥ˥奢ʤɡˤˡ嵭
 *        ɽѾ浪Ӳ̵ݾڵǺܤ뤳ȡ
 *    (b) ۤη֤̤ˡˤäơTOPPERSץȤ
 *        𤹤뤳ȡ
 *  (4) ܥեȥѤˤľŪޤϴŪ뤤ʤ»
 *      ⡤嵭ԤTOPPERSץȤդ뤳ȡ
 *      ޤܥեȥΥ桼ޤϥɥ桼Τʤ
 *      ͳ˴Ťᤫ⡤嵭ԤTOPPERSץȤ
 *      դ뤳ȡ
 *
 *  ܥեȥϡ̵ݾڤ󶡤ƤΤǤ롥嵭Ԥ
 *  TOPPERSץȤϡܥեȥ˴ؤơλŪ
 *  ФŬޤơʤݾڤԤʤޤܥեȥ
 *  ѤˤľŪޤϴŪʤ»˴ؤƤ⡤
 *  Ǥʤ
 *
 */
#ifndef TOPPERS_CHIP_GTIMER_H
#define TOPPERS_CHIP_GTIMER_H

#include <sil.h>
#include "mpcore.h"

/*
 *  Global Timer 32bitΤߤ
 */

/*
 *  Global Timer interrupts number, priority and attributes
 */
#define INHNO_TIMER_PRC1     (0x00010000 | DIC_IRQNO_GTM)
#define INHNO_TIMER_PRC2     (0x00020000 | DIC_IRQNO_GTM)
#define INHNO_TIMER_PRC3     (0x00030000 | DIC_IRQNO_GTM)
#define INHNO_TIMER_PRC4     (0x00040000 | DIC_IRQNO_GTM)

#define INTNO_TIMER_PRC1     (0x00010000 | DIC_IRQNO_GTM)
#define INTNO_TIMER_PRC2     (0x00020000 | DIC_IRQNO_GTM)
#define INTNO_TIMER_PRC3     (0x00030000 | DIC_IRQNO_GTM)
#define INTNO_TIMER_PRC4     (0x00040000 | DIC_IRQNO_GTM)


#define INTATR_TIMER   0U

#ifndef TOPPERS_MACRO_ONLY

/*
 *  Type definition for the internal representation of the timer value
 */
typedef uint32_t CLOCK;

/*
 *  Convertion between the internal representation of the timer value
 *  and miliseconds/microseconds. One tick equals to 1ms.
 */
#define TIMER_CLOCK          MPCORE_GTM_LOAD_1MS // cycles
#define TO_CLOCK(nume, deno) (TIMER_CLOCK * (nume) / (deno))
#define TO_USEC(clock)       (((SYSUTM) clock) * 1000U / TIMER_CLOCK)

/*
 *  Maximum timer value (internal representation in cycles)
 */
#define MAX_CLOCK        ((CLOCK) 0xffffffffU)

/*
 *  Approximate amount of time since the timer interrupt occurred
 */
#define GET_TOLERANCE    100U    /* delay tolerance (internal represent.) */
#define BEFORE_IREQ(clock) \
((clock) >= TO_CLOCK(TIC_NUME, TIC_DENO) - GET_TOLERANCE)

/*
 *  Timer initialization
 */
extern void target_timer_initialize(intptr_t exinf);

/*
 * Timer termination
 */
extern void target_timer_terminate(intptr_t exinf);

/*
 *  Get the current timer value
 */
Inline CLOCK
target_timer_get_current(void)
{
	uint32_t count_l, count_h_p, count_h;
	uint64_t count;
	uint32_t comp_l, comp_h;
	uint64_t comp;
	uint32_t cyc = TO_CLOCK(TIC_NUME, TIC_DENO);
    
	/*
	 * 32bit2Ĥ64bit󥿤¸Ƥ뤿ᡤ
	 * 夲θɤ߹ߤȤ롥
	 */
	count_h_p = sil_rew_mem((void *)MPCORE_GTM_COUNT_U);
	count_l = sil_rew_mem((void *)MPCORE_GTM_COUNT_L);
	count_h = sil_rew_mem((void *)MPCORE_GTM_COUNT_U);
	if (count_h_p != count_h) {
		count_l = sil_rew_mem((void *)MPCORE_GTM_COUNT_L);
	}

	count = count_h;
	count = count << 32;
	count += count_l;

	comp_l = sil_rew_mem((void *)MPCORE_GTM_COMPV_L);
	comp_h = sil_rew_mem((void *)MPCORE_GTM_COMPV_U);    
	comp = comp_h;
	comp = comp << 32;
	comp += comp_l;

	return((uint32_t)(cyc - (comp - count)));
}

/*
 *  Check for pending timer interrupts
 */
Inline bool_t
target_timer_probe_int(void)
{
	if((sil_rew_mem((void *)MPCORE_GTM_ISR) & MPCORE_GTM_ISR_EF) == MPCORE_GTM_ISR_EF) {
		return true;
	} else {
		return false;
	}
}

/*
 *  Timer interrupt handler
 */
extern void target_timer_handler(void);

#endif /* TOPPERS_MACRO_ONLY */
#endif /* TOPPERS_CHIP_TIMER_H */
