/*
 *  TOPPERS/FMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Flexible MultiProcessor Kernel    
 * 
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 *  Copyright (C) 2007-2015 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN    
 * 
 *  嵭Ԥϡʲ(1)(4)ξ˸¤ꡤܥեȥ
 *  ܥեȥѤΤޤࡥʲƱˤѡʣ
 *  ѡۡʰʲѤȸƤ֡ˤ뤳Ȥ̵ǵ롥
 *  (1) ܥեȥ򥽡ɤηѤˤϡ嵭
 *      ɽѾ浪Ӳ̵ݾڵ꤬Τޤޤηǥ
 *      ˴ޤޤƤ뤳ȡ
 *  (2) ܥեȥ򡤥饤֥ʤɡ¾Υեȥȯ˻
 *      ѤǤǺۤˤϡۤȼɥȡ
 *      ԥޥ˥奢ʤɡˤˡ嵭ɽѾ浪Ӳ
 *      ̵ݾڵǺܤ뤳ȡ
 *  (3) ܥեȥ򡤵Ȥ߹ʤɡ¾Υեȥȯ˻
 *      ѤǤʤǺۤˤϡΤ줫ξ
 *      ȡ
 *    (a) ۤȼɥȡѼԥޥ˥奢ʤɡˤˡ嵭
 *        ɽѾ浪Ӳ̵ݾڵǺܤ뤳ȡ
 *    (b) ۤη֤̤ˡˤäơTOPPERSץȤ
 *        𤹤뤳ȡ
 *  (4) ܥեȥѤˤľŪޤϴŪ뤤ʤ»
 *      ⡤嵭ԤTOPPERSץȤդ뤳ȡ
 *      ޤܥեȥΥ桼ޤϥɥ桼Τʤ
 *      ͳ˴Ťᤫ⡤嵭ԤTOPPERSץȤ
 *      դ뤳ȡ
 * 
 *  ܥեȥϡ̵ݾڤ󶡤ƤΤǤ롥嵭Ԥ
 *  TOPPERSץȤϡܥեȥ˴ؤơλŪ
 *  ФŬޤơʤݾڤԤʤޤܥեȥ
 *  ѤˤľŪޤϴŪʤ»˴ؤƤ⡤
 *  Ǥʤ
 * 
 *  @(#) $Id: ns16550.c 1019 2013-07-11 13:29:21Z ertl-honda $
 */

/*
 *     IMXUART  ʰSIOɥ饤
 */
#include <kernel.h>
#include <t_syslog.h>
#include <sil.h>
#include "target_syssvc.h"
#include "imxuart.h"

#define UART_URXD  (0x00)
#define UART_UTXD  (0x40)
#define UART_UCR1  (0x80)
#define UART_UCR2  (0x84)
#define UART_UCR3  (0x88)
#define UART_UCR4  (0x8C)
#define UART_UFCR  (0x90)
#define UART_USR1  (0x94)
#define UART_USR2  (0x98)
#define UART_UESC  (0x9C)
#define UART_UTIM  (0xA0)
#define UART_UBIR  (0xA4)
#define UART_UBMR  (0xA8)
#define UART_UBRC  (0xAC)
#define UART_ONEMS (0xB0)
#define UART_UTS   (0xB4)
#define UART_UMCR  (0xB8)

#define UART_UTS_TXFULL (1<<4)
#define UART_UTS_RXEMPTY (1<<5)

#define UART_UCR1_TXMPTYEN (1<<6)
#define UART_UCR1_RRDYEN   (1<<9)
#define UART_UCR1_UARTEN   (1<<0)

#define UART_UCR2_IRTS   (1<<14)
#define UART_UCR2_PREN   (1<<8)
#define UART_UCR2_STPB   (1<<6)
#define UART_UCR2_WS     (1<<5)
#define UART_UCR2_TXEN   (1<<2)
#define UART_UCR2_RXEN   (1<<1)
#define UART_UCR2_SRST   (1<<0)

#define UART_USR2_TXFE   (1<<14)


/*
 *  Хå롼μֹ
 */
#define SIO_RDY_SND	1U		/* ǽХå */
#define SIO_RDY_RCV	2U		/* ΥХå */

/*
 *  ꥢI/OݡȽ֥å
 */
typedef struct sio_port_initialization_block {
	uint32_t reg_base;    /* 쥸Υ١ɥ쥹 */
	uint8_t intno;        /* ֹ */
} SIOPINIB;

/*
 *  ꥢI/Oݡȴ֥å
 */
struct sio_port_control_block {
	const SIOPINIB	*p_siopinib; /* ꥢI/OݡȽ֥å */
	intptr_t		exinf;      /* ĥ */
	bool_t			openflag;   /* ץѤߥե饰 */
	bool_t			sendflag;   /* ߥ֥͡ե饰 */
	bool_t			getready;   /* ʸ */
	bool_t			putready;   /* ʸǤ */
};


/*
 * ꥢI/OݡȽ֥å
 */
const SIOPINIB siopinib_table[TNUM_SIOP] = {
#ifdef TARGET_USE_UART1
	{UART1_BASE, DIC_IRQNO_UART1},
#elif defined(TARGET_USE_UART2)
	{UART2_BASE, DIC_IRQNO_UART2},
#elif defined(TARGET_USE_UART3)
	{UART3_BASE, DIC_IRQNO_UART3},
#endif /* TARGET_USE_UART1 */
};

/*
 *  ꥢI/OݡȽ֥åμФ
 */
#define INDEX_SIOPINIB(siopid)  ((uint_t)((siopid) - 1))
#define get_siopinib(siopid)  (&(siopinib_table[INDEX_SIOPINIB(siopid)]))

/*
 *  ꥢI/Oݡȴ֥åΥꥢ
 */
SIOPCB  siopcb_table[TNUM_SIOP];

/*
 *  ꥢI/OݡID֥åФΥޥ
 */
#define INDEX_SIOP(siopid)  ((uint_t)((siopid) - 1))
#define get_siopcb(siopid)  (&(siopcb_table[INDEX_SIOP(siopid)]))


/*
 * ʸ?
 */ 
Inline bool_t
imxuart_getready(SIOPCB *p_siopcb)
{
	uint16_t status;

	status = sil_rew_mem((void*)(p_siopcb->p_siopinib->reg_base + UART_UTS));
	return(!(status & UART_UTS_RXEMPTY));
}

/*
 * ʸǤ뤫?
 */
Inline bool_t
imxuart_putready(SIOPCB *p_siopcb)
{
	uint16_t status;

	status = sil_rew_mem((void*)(p_siopcb->p_siopinib->reg_base + UART_UTS));
	return(!(status & UART_UTS_TXFULL));
}

/*
 *  ʸμФ
 */
Inline uint8_t
imxuart_getchar(SIOPCB *p_siopcb)
{
	return(sil_rew_mem((void*)(p_siopcb->p_siopinib->reg_base + UART_URXD)));
}

/*
 *  ʸν񤭹
 */
Inline void
imxuart_putchar(SIOPCB *p_siopcb, uint8_t c)
{
	sil_wrw_mem((void*)(p_siopcb->p_siopinib->reg_base + UART_UTXD), c);
}

/*
 *  ߵ
 */
Inline void
imxuart_enable_send(SIOPCB *p_siopcb)
{
	sil_wrw_mem((void*)(p_siopcb->p_siopinib->reg_base + UART_UCR1),
				sil_rew_mem((void*)(p_siopcb->p_siopinib->reg_base + UART_UCR1))|UART_UCR1_TXMPTYEN);
}

/*
 *  ߶ػ
 */
Inline void
imxuart_disable_send(SIOPCB *p_siopcb)
{
	sil_wrw_mem((void*)(p_siopcb->p_siopinib->reg_base + UART_UCR1),
				sil_rew_mem((void*)(p_siopcb->p_siopinib->reg_base + UART_UCR1))&~UART_UCR1_TXMPTYEN);
}

/*
 *  ߵ
 */
Inline void
imxuart_enable_rcv(SIOPCB *p_siopcb)
{
	sil_wrw_mem((void*)(p_siopcb->p_siopinib->reg_base + UART_UCR1),
				sil_rew_mem((void*)(p_siopcb->p_siopinib->reg_base + UART_UCR1))|UART_UCR1_RRDYEN);
}

/*
 *  ߶ػ
 */
Inline void
imxuart_disable_rcv(SIOPCB *p_siopcb)
{
	sil_wrw_mem((void*)(p_siopcb->p_siopinib->reg_base + UART_UCR1),
				sil_rew_mem((void*)(p_siopcb->p_siopinib->reg_base + UART_UCR1))&~UART_UCR1_RRDYEN);
}

/*
 *  ꥢI/OݡȤؤΥݡ󥰤Ǥν
 */
void
sio_pol_putc(char c, ID siopid)
{
	const SIOPINIB *p_siopinib;

	p_siopinib = get_siopinib(siopid);

	while((sil_rew_mem((void*)(p_siopinib->reg_base + UART_UTS)) & UART_UTS_TXFULL));
	sil_wrw_mem((void*)(p_siopinib->reg_base + UART_UTXD), c);
}

/*
 *  SIOɥ饤Фν롼
 */
void
sio_initialize(intptr_t exinf)
{
	SIOPCB  *p_siopcb;
	uint_t  i;

	/*
	 *  ꥢI/Oݡȴ֥åν
	 */
	for (p_siopcb = siopcb_table, i = 0; i < TNUM_SIOP; p_siopcb++, i++) {
		p_siopcb->p_siopinib = &(siopinib_table[i]);
		p_siopcb->openflag = false;
		p_siopcb->sendflag = false;
	} 
}

    
/*
 * SIOPINIBν
 */
static void
imxuart_init_siopinib(const SIOPINIB *p_siopinib)
{
	/* Wait until Tx FIFO empty */
	if (sil_rew_mem((void*)(p_siopinib->reg_base + UART_UCR1)) & UART_UCR1_UARTEN) {
		while(!(sil_rew_mem((void*)(p_siopinib->reg_base + UART_USR2)) & UART_USR2_TXFE));
	}

	/* Disable */
	sil_wrw_mem((void*)(p_siopinib->reg_base + UART_UCR1), 0x00);

	/* Disable tranmitter/receiver, Software Reset */
	sil_wrw_mem((void*)(p_siopinib->reg_base + UART_UCR2),0x00);

	while((sil_rew_mem((void*)(p_siopinib->reg_base + UART_UCR2)) & UART_UCR2_SRST) != UART_UCR2_SRST);

	/* Set RXD Muxed Input Selected */
	sil_wrw_mem((void*)(p_siopinib->reg_base + UART_UCR3), 0x0784);

	/* CTS Trigger Level */
	sil_wrw_mem((void*)(p_siopinib->reg_base + UART_UCR4), 0x8000);

	/* Set Escape Character */
	sil_wrw_mem((void*)(p_siopinib->reg_base + UART_UESC), 0x002b);
	/* Set Escape Timer */
	sil_wrw_mem((void*)(p_siopinib->reg_base + UART_UTIM), 0x0000);

	/* Receive FIFO interrupt trigger level */
	sil_wrw_mem((void*)(p_siopinib->reg_base + UART_UFCR),
				(sil_rew_mem((void*)(p_siopinib->reg_base + UART_UFCR)) & ~0x3f) |0x01);

	/* Transmit FIFO interrupt trigger level */
	sil_wrw_mem((void*)(p_siopinib->reg_base + UART_UFCR),
				(sil_rew_mem((void*)(p_siopinib->reg_base + UART_UFCR)) & ~0xfc00) | (0x02<<10));

	/*
	 * Baud rateϢ(u-bootǤPLL꤬Ѳɽɬפ)
	 */
	/* Rererence Frequency Divider by 1*/
	sil_wrw_mem((void*)(p_siopinib->reg_base + UART_UFCR),
				(sil_rew_mem((void*)(p_siopinib->reg_base + UART_UFCR)) & ~0x0380) | (0x04<<7));
	/* Set Baud rate */
	sil_wrw_mem((void*)(p_siopinib->reg_base + UART_UBIR), 0x000f);
	sil_wrw_mem((void*)(p_siopinib->reg_base + UART_UBMR), 0x015b);

	/* 8bit, 1stop, Noprity, Ignore RTS, Enable tranmitter/receiver */
	sil_wrw_mem((void*)(p_siopinib->reg_base + UART_UCR2), UART_UCR2_IRTS|UART_UCR2_WS|UART_UCR2_TXEN|UART_UCR2_RXEN|UART_UCR2_SRST);
	
	/* Enable */
	sil_wrw_mem((void*)(p_siopinib->reg_base + UART_UCR1), UART_UCR1_UARTEN);
}


/*
 *  ưΥ٥Ѥν
 */
void
sio_low_init(ID siopid)
{
	const SIOPINIB *p_siopinib;

	p_siopinib = get_siopinib(siopid);
	imxuart_init_siopinib(p_siopinib);
}

/*
 * ꥢI/OݡȤΥץ
 */
SIOPCB *
sio_opn_por(ID siopid, intptr_t exinf)
{
	SIOPCB  *p_siopcb;
	bool_t  opnflg;
	ER      ercd;

	p_siopcb = get_siopcb(siopid);

	/*
	 *  ץ󤷤ݡȤ뤫opnflgɤǤ
	 */
	opnflg = p_siopcb->openflag;
    
	/*
	 * 
	 */
	imxuart_init_siopinib(p_siopcb->p_siopinib);
    
	/* ߶ػ */
	imxuart_enable_rcv(p_siopcb);
	/* ߶ػ */
	imxuart_disable_rcv(p_siopcb);
    
	p_siopcb->exinf = exinf;
	p_siopcb->getready = p_siopcb->putready = false;
	p_siopcb->openflag = true;

	/*
	 *  ꥢI/OߤΥޥ롥
	 */
	if (!opnflg) {
		ercd = ena_int(p_siopcb->p_siopinib->intno);
		assert(ercd == E_OK);
	}

	return(p_siopcb);
}

/*
 *  ꥢI/OݡȤΥ
 */
void
sio_cls_por(SIOPCB *p_siopcb)
{
	/* ߶ػ */
	imxuart_disable_send(p_siopcb);
	imxuart_disable_rcv(p_siopcb);
	p_siopcb->openflag = false;

	/*
	 *  ꥢI/Oߤޥ롥
	 */
	if (!(p_siopcb->openflag)) {
		dis_int(p_siopcb->p_siopinib->intno);
	}
}

/*
 *  ꥢI/OݡȤؤʸ
 */
bool_t
sio_snd_chr(SIOPCB *p_siopcb, char c)
{
	if (imxuart_putready(p_siopcb)){
		imxuart_putchar(p_siopcb, c);
		return(true);
	}
	return(false);
}

/*
 *  ꥢI/OݡȤʸ
 */
int
sio_rcv_chr(SIOPCB *p_siopcb)
{
	if (imxuart_getready(p_siopcb)) {
		return((int)(uint8_t)imxuart_getchar(p_siopcb));
	}
	return(-1);
}

/*
 *  ꥢI/OݡȤΥХåε
 */
void
sio_ena_cbr(SIOPCB *p_siopcb, uint_t cbrtn)
{
	switch (cbrtn) {
	  case SIO_RDY_SND:
		imxuart_enable_send(p_siopcb);
		break;
	  case SIO_RDY_RCV:
		imxuart_enable_rcv(p_siopcb);
		break;
	}
}

/*
 *  ꥢI/OݡȤΥХåζػ
 */
void
sio_dis_cbr(SIOPCB *p_siopcb, uint_t cbrtn)
{
	switch (cbrtn) {
	  case SIO_RDY_SND:
		imxuart_disable_send(p_siopcb);
		break;
	  case SIO_RDY_RCV:
		imxuart_disable_rcv(p_siopcb);
		break;
	}
}

/*
 *  ꥢI/OݡȤФ߽
 */
static void
imxuart_isr_siop(SIOPCB *p_siopcb)
{
	/* ߤΥꥢ */
	sil_wrw_mem((void*)(p_siopcb->p_siopinib->reg_base + UART_USR1),
				sil_rew_mem((void*)(p_siopcb->p_siopinib->reg_base + UART_USR1)));
	sil_wrw_mem((void*)(p_siopcb->p_siopinib->reg_base + UART_USR2),
				sil_rew_mem((void*)(p_siopcb->p_siopinib->reg_base + UART_USR2)));

	if (imxuart_getready(p_siopcb)) {
		/*
		 *  ΥХå롼ƤӽФ
		 */
		sio_irdy_rcv(p_siopcb->exinf);
	}else if (imxuart_putready(p_siopcb)) {
		/*
		 *  ǽХå롼ƤӽФ
		 */
		sio_irdy_snd(p_siopcb->exinf);
	}else{
		sio_irdy_snd(p_siopcb->exinf);
	}
}


/*
 *  SIOγߥӥ롼
 */
void
sio_isr(intptr_t exinf)
{
	imxuart_isr_siop(get_siopcb(exinf));
}
