/*
 *  TOPPERS/FMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Flexible MultiProcessor Kernel
 *
 *  Copyright (C) 2007-2015 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 *
 *  嵭Ԥϡʲ(1)(4)ξ˸¤ꡤܥեȥ
 *  ܥեȥѤΤޤࡥʲƱˤѡʣ
 *  ѡۡʰʲѤȸƤ֡ˤ뤳Ȥ̵ǵ롥
 *  (1) ܥեȥ򥽡ɤηѤˤϡ嵭
 *      ɽѾ浪Ӳ̵ݾڵ꤬Τޤޤηǥ
 *      ˴ޤޤƤ뤳ȡ
 *  (2) ܥեȥ򡤥饤֥ʤɡ¾Υեȥȯ˻
 *      ѤǤǺۤˤϡۤȼɥȡ
 *      ԥޥ˥奢ʤɡˤˡ嵭ɽѾ浪Ӳ
 *      ̵ݾڵǺܤ뤳ȡ
 *  (3) ܥեȥ򡤵Ȥ߹ʤɡ¾Υեȥȯ˻
 *      ѤǤʤǺۤˤϡΤ줫ξ
 *      ȡ
 *    (a) ۤȼɥȡѼԥޥ˥奢ʤɡˤˡ嵭
 *        ɽѾ浪Ӳ̵ݾڵǺܤ뤳ȡ
 *    (b) ۤη֤̤ˡˤäơTOPPERSץȤ
 *        𤹤뤳ȡ
 *  (4) ܥեȥѤˤľŪޤϴŪ뤤ʤ»
 *      ⡤嵭ԤTOPPERSץȤդ뤳ȡ
 *      ޤܥեȥΥ桼ޤϥɥ桼Τʤ
 *      ͳ˴Ťᤫ⡤嵭ԤTOPPERSץȤ
 *      դ뤳ȡ
 *
 *  ܥեȥϡ̵ݾڤ󶡤ƤΤǤ롥嵭Ԥ
 *  TOPPERSץȤϡܥեȥ˴ؤơλŪ
 *  ФŬޤơʤݾڤԤʤޤܥեȥ
 *  ѤˤľŪޤϴŪʤ»˴ؤƤ⡤
 *  Ǥʤ
 *
 *  @(#) $Id: target_config.c 1048 2013-11-13 10:29:14Z ertl-honda $  
 */

#include "kernel_impl.h"
#include <sil.h>
#include "imxuart.h"
#include "pl310.h"
#include "mpcore.h"

#if defined(TOPPERS_SAFEG_SECURE) | defined(TOPPERS_SAFEG_NONSECURE)
#include "syscalls_api.h"
#endif /* defined(TOPPERS_SAFEG_SECURE) | defined(TOPPERS_SAFEG_NONSECURE) */

/*
 *  entry point (start.S)
 */
extern void start(void);

/*
 *  Initialize Diag Register (target_support.S)
 */
extern void init_diag_reg(void);

/*
 *  
 */
static void init_uart_iomuxc(void);

#ifdef TOPPERS_SAFEG_SECURE
/*
 * diagnostic registerΥ0¾ؤΥԡѿ
 */
static uint32_t diag_reg_val;

void
init_tzasc(void){

	/*  In case of Illegal access : Error */
	sil_wrw_mem((void*)0x021D0004,1);
	
	/*
	 * Region 0 : All DDR
	 *          : T Read Write : NT Read Write
	 */
	sil_wrw_mem((void*)0x021D0108,0xf0000000);

	/*
	 * Region 1 : 0x4f000000 - 0x47ffffff : 8MB
	 *          : T Read Write
	 */
	sil_wrw_mem((void*)0x021D0110,(0x4f000000));
	sil_wrw_mem((void*)0x021D0118,0xc000002d);

	/*
	 * Region 2 : 0x4f000000 - 0x4f07ffff : 512K
	 *          : T Read
	 *          : SafeG Code
	 */
	sil_wrw_mem((void*)0x021D0120,(0x4f000000));
	sil_wrw_mem((void*)0x021D0128,0x80000001|(0x12<<1));

	/*
	 * Region 2 : 0x4f100000 - 0x4f1fffff : 1M
	 *          : T Read
	 *          : FMP Code
	 */
	sil_wrw_mem((void*)0x021D0130,0x4f100000);
	sil_wrw_mem((void*)0x021D0138,0x80000027);
}

#endif /* TOPPERS_SAFEG_SECURE */

/*
 *  Master processor initialization before str_ker().
 *  (E.g.: initialize the interrupt controller or the SCU.)
 */
void
target_mprc_initialize(void)
{
#if !defined(TOPPERS_SAFEG_NONSECURE)
	/*
	 * SafeG ̤ѻ/SafeG ѻ(Secure¦)
	 */

	/*
	 * U-boot may enable L1 and L2.
	 */
	pl310_disable();
	mpcore_cache_disable();
#endif /* !defined(TOPPERS_SAFEG_NONSECURE) */

	chip_mprc_initialize();

#if !defined(TOPPERS_SAFEG_SECURE) & !defined(TOPPERS_SAFEG_NONSECURE)
	/*
	 * SafeG ̤ѻ
	 */
#if TNUM_PRCID >= 2
	/* Wake up processor 2 */
	sil_wrw_mem((uint32_t*)SRC_GPR3, (uint32_t)start);
	sil_wrw_mem((uint32_t*)SRC_SCR,
				sil_rew_mem((uint32_t*)SRC_SCR)|SRC_SCR_CORE1_ENA);
#endif /* TNUM_PRCID >= 2 */
#if TNUM_PRCID >= 3
	/* Wake up processor 3 */
	sil_wrw_mem((uint32_t*)SRC_GPR5, (uint32_t)start);
	sil_wrw_mem((uint32_t*)SRC_SCR,
				sil_rew_mem((uint32_t*)SRC_SCR)|SRC_SCR_CORE2_ENA);
#endif /* TNUM_PRCID >= 3 */
#if TNUM_PRCID >= 4
	/* Wake up processor 4 */
	sil_wrw_mem((uint32_t*)SRC_GPR7, (uint32_t)start);
	sil_wrw_mem((uint32_t*)SRC_SCR,
				sil_rew_mem((uint32_t*)SRC_SCR)|SRC_SCR_CORE3_ENA);
#endif /* TNUM_PRCID >= 4 */
#endif /* !defined(TOPPERS_SAFEG_SECURE) & !defined(TOPPERS_SAFEG_NONSECURE) */

#ifdef TOPPERS_SAFEG_NONSECURE
	/*
	 *  SafeG ѻ(NonSecure¦) 
	 *  ץå2,3,4Secure¦OSRAM˥ԡ first_switch_to_nt
	 *   SRC_SPRX ꤵΤԤäƤ뤿ᡤ
	 *  ư뤿˽񤭹ࡥ
	 */
#if TNUM_PRCID >= 2
	sil_wrw_mem((uint32_t*)SRC_GPR3, (uint32_t)start);
#endif /* TNUM_PRCID >= 2 */
#if TNUM_PRCID >= 3
	sil_wrw_mem((uint32_t*)SRC_GPR5, (uint32_t)start);
#endif /* TNUM_PRCID >= 3 */
#if TNUM_PRCID >= 4
	sil_wrw_mem((uint32_t*)SRC_GPR7, (uint32_t)start);
#endif /* TNUM_PRCID >= 4 */
#endif /* TOPPERS_SAFEG_NONSECURE */

#ifdef TOPPERS_SAFEG_SECURE
	/*
	 *  SafeG ѻ(Secure¦)
	 */

	/*
	 *  TZASCν
	 */
	init_tzasc();

	/* Set Non-Secure Core Address */
	sil_wrw_mem((uint32_t*)SRC_GPR1, (uint32_t)TARGET_PRC1_NS_START_ADDR);
	/* Set NULL(0x00) for Linux 2,3,4 core boot */
#if TNUM_PRCID >= 2
	sil_wrw_mem((uint32_t*)SRC_GPR3, 0x00);
#endif /* TNUM_PRCID >= 2 */
#if TNUM_PRCID >= 3
	sil_wrw_mem((uint32_t*)SRC_GPR5, 0x00);
#endif /* TNUM_PRCID >= 3 */
#if TNUM_PRCID >= 4
	sil_wrw_mem((uint32_t*)SRC_GPR7, 0x00);
#endif /* TNUM_PRCID >= 4 */

	/*
	 * Linuxdiagnostic register˥ӥåȥåȤƤ
	 * åβͭˤʤ롥
	 * Υ쥸ϳƥˤ뤬֡ȥˤäơ
	 * 0ˤåȤʤᡤ¾Υ˥ԡ뤿
	 * ö쥸¸롥
	 */
	init_diag_reg();
	Asm("mrc p15, 0, %0, c15, c0, 1":"=r" (diag_reg_val) ::"cc");
#endif /* TOPPERS_SAFEG_SECURE */
}

void
target_fput_log(char c)
{
	if (c == '\n') {
		sio_pol_putc('\r', 1);
	}
		sio_pol_putc(c, 1);
}

/*
 * MMU initialization (called from the chip code)
 *
 * We create a simpe 1-to-1 (physical address = virtual address) memory
 * mapping and enable caching for the DDR memory.
 */
void
target_mmu_init(void)
{
	MEMORY_ATTRIBUTE m_attribute;

	/*
	 * Basic initialization of the whole virtual memory space to
	 * non-cachable, non-bufferable, strongly-ordered.
	 */
	m_attribute.pa   = 0x00000000;
	m_attribute.va   = m_attribute.pa;
	m_attribute.size = 0x80000000; /* 2GB */
	m_attribute.ns   = 0;          /* 0=Secure */
	m_attribute.s    = 1;          /* 1=Shared */
	m_attribute.ap   = 3;          /* Full access */
	m_attribute.tex  = 0;          /* Strongly Ordered */
	m_attribute.c    = 0;
	m_attribute.b    = 0;

	mmu_map_memory (&m_attribute);

	m_attribute.pa   = 0x80000000; /* 2GB */
	m_attribute.va   = m_attribute.pa;

	mmu_map_memory (&m_attribute);

	/*
	 *  Enable caching for the DDR
	 */
	m_attribute.pa   = DDR_ADDR;
	m_attribute.va   = m_attribute.pa;
	m_attribute.size = DDR_SIZE;
	m_attribute.ns   = 0;      /* Secure   */
	m_attribute.s    = 1;      /* 1=Shared */
	m_attribute.ap   = 3;      /* Full access */
	/* Outer and Inner Write-Back, Write-Allocate */
	m_attribute.tex  = 1;             
	m_attribute.c    = 1;             
	m_attribute.b    = 1;             

	mmu_map_memory(&m_attribute);

#ifdef TOPPERS_SAFEG_SECURE

	/*
	 *  Enable caching for the DDR
	 */
	m_attribute.pa   = DDR_ADDR;
	m_attribute.va   = m_attribute.pa;
	m_attribute.size = 0x3F000000;
	m_attribute.ns   = 1;        /* NonSecure    */
	m_attribute.s    = 1;        /* 1=Shared */
	m_attribute.ap   = 3;        /* Full access */

	/*
	 * ǤʤȡL2νΤLinuxidݤSafeG
	 * Linuxѿ˽񤭹ͤȿǤʤ.
	 * L2ͭͭ׸Ƥ
	 */
	
	/* åѿξϤư */
	/* Onter and Inner Write-Back; no Write-Allocate ng */ 
	m_attribute.tex  = 0;
	m_attribute.c    = 1;
	m_attribute.b    = 1;

	/* ХѿξϤư */
	/* Outer and Inner Write-Through, no Write-Allocate */
//	m_attribute.tex  = 0;
//	m_attribute.c    = 1;
//	m_attribute.b    = 0;

	mmu_map_memory(&m_attribute);

	/*
	 * SafeG-COM shared memory area
	 */
	m_attribute.pa   = SAFEG_COM_MEM_ADDR;
	m_attribute.va   = m_attribute.pa;
	m_attribute.size = SAFEG_COM_MEM_SIZE * 1024 * 1024;
	m_attribute.ns   = 1;          /* NonSecure    */
	m_attribute.s    = 1;          /* ͭ     */
	m_attribute.ap   = 3;          /* ե륢 */
	m_attribute.tex  = 0;          /* åON */
	m_attribute.c    = 1;          /* Outer and Inner */          
	m_attribute.b    = 1;          /* Inner Write-Back, Write Allocate */

	mmu_map_memory(&m_attribute);

#endif /* TOPPERS_SAFEG_SECURE */
}

/*
 *  Vector Tablechip_support.S
 */
extern void *vector_table;

/*
 *  L2åν
 */
static void l2cache_init(void)
{
#ifdef TOPPERS_SAFEG_NONSECURE
	char l2init_id_syscall_name[] = "l2init";
	uint32_t l2init_id = 0;
	uint32_t err;
	err = safeg_syscall_getid(l2init_id_syscall_name, &l2init_id);
	safeg_syscall_invoke(l2init_id, 0, 0, 0);
#else /* !TOPPERS_SAFEG_NONSECURE */
	uint32_t aux_ctrl = 0;
	pl310_init(aux_ctrl, ~0x0U);
#endif /* TOPPERS_SAFEG_NONSECURE */
}

#ifdef TOPPERS_SAFEG_SECURE

uint32_t
safeg_l2cache_init(const uint32_t core_id,
				   const uint32_t ns,
				   const uint32_t a,
				   const uint32_t b,
				   const uint32_t c)
{
	l2cache_init();
	return SAFEG_OK(SAFEG_SYSCALL_RET__DONT_SWITCH);
}

struct safeg_syscall l2init_call = {
	.is_t_callable  = 1,
	.is_nt_callable = 1,
	.name = "l2init",
	.function = safeg_l2cache_init,
};

/*
 *  L2åǥХå쥸񤭹
 */
uint32_t
l2cache_debug_set(const uint32_t core_id,
				  const uint32_t ns,
				  const uint32_t a,
				  const uint32_t b,
				  const uint32_t c)
{
	pl310_debug_set(a);
	return SAFEG_OK(SAFEG_SYSCALL_RET__DONT_SWITCH);
}

struct safeg_syscall l2debug_set_call = {
	.is_t_callable  = 1,
	.is_nt_callable = 1,
	.name = "l2debug",
	.function = l2cache_debug_set,
};

#endif /* TOPPERS_SAFEG_SECURE */

#if !defined(TOPPERS_SAFEG_SECURE)
volatile uint32_t target_initialize_end[TNUM_PRCID];
#endif /* !TOPPERS_SAFEG_SECURE */

void
target_initialize(void)
{
	/* Disbale high vector */
	set_high_vector(false);

	/* Set Vector Table */
	CP15_SET_VBAR((uint32_t) &vector_table);

	/*
	 * Initialize chip
	 */
	chip_initialize();

#ifdef TOPPERS_SAFEG_SECURE
	if (x_sense_mprc()) {
		/*
		 * L2åؿSafeGϿ
		 */
		uint32_t id;
		safeg_syscall_regdyn(&l2init_call, &id);

		/*
		 * L2åǥХå쥸񤭹ߴؿSafeGϿ
		 */
		safeg_syscall_regdyn(&l2debug_set_call, &id);
	}
	else {
		/* diagnostic registerؤν񤭹 */
		Asm("mcr p15, 0, %0, c15, c0, 1"::"r"(diag_reg_val) :"cc");
	}
#endif /*  TOPPERS_SAFEG_SECURE */

	/*
	 *  Initialize the UART. If we are using a
	 *  global logger (shared by all cores) only
	 *  the master processor (mprc) initializes it.
	 */
#ifdef G_SYSLOG
	if (x_sense_mprc()) {
		init_uart_iomuxc();
		sio_low_init(1);
	}
#else /* G_SYSLOG */
#error Only support G_SYSLOG
#endif /* G_SYSLOG */

#if !defined(TOPPERS_SAFEG_SECURE)
	/* Enable L2 cache by myself in case of without safeg or with safeg Non-Secure */
	if (x_sense_mprc()) {
		uint32_t i;
		uint32_t sum = 0;
		while(1) {
			for(i = 1; i < TNUM_PRCID; i++) {
				sum += target_initialize_end[i];
			}
			if (sum == (TNUM_PRCID - 1)) {
				break;
			}
		}
		l2cache_init();
	}
	target_initialize_end[x_prc_index()] = 1;
#endif /* !TOPPERS_SAFEG_SECURE */
}

/*
 *  Target-dependent exit processing
 */
void
target_exit(void)
{
	/*
	 *  Chip exit processing
	 */
	chip_exit();

	/*
	 *  Endless loop
	 */
	while(1);
}

static void
set_iomuxc(uint32_t mux_ctl_offset, uint32_t mux_mode_val,
			uint32_t sel_input_offset, uint32_t sel_input_val,
			uint32_t pad_ctl_offset, uint32_t pad_ctl_val)
{
	if (mux_ctl_offset)
	  sil_wrw_mem((uint32_t*)(IOMUXC_BASE + mux_ctl_offset), mux_mode_val);

	if (sel_input_offset)
	  sil_wrw_mem((uint32_t*)(IOMUXC_BASE + sel_input_offset), sel_input_val);

	if (pad_ctl_offset)
	  sil_wrw_mem((uint32_t*)(IOMUXC_BASE + pad_ctl_offset), pad_ctl_val);
}


#define UART_PAD_CTL_VAL (IOMUXC_PAD_CTL_PUS_100K_UP | IOMUXC_PAD_CTL_SPEED_MED | IOMUXC_PAD_CTL_DSE_40ohm | IOMUXC_PAD_CTL_SRE_FAST | IOMUXC_PAD_CTL_HYS)

static void
init_uart_iomuxc(void){
#ifdef TARGET_USE_UART1
	/* For FreeScale Sabre */
	/* UART1 TX : CSI0_DAT10 */
	set_iomuxc(0x0280,
			   3,
			   0x0000,
			   0,
			   0x0650,
			   UART_PAD_CTL_VAL);
	/* UART1 RX : CSI0_DAT11 */
	set_iomuxc(0x0284,
			   3,
			   0x0920,
			   1,
			   0x0654,
			   UART_PAD_CTL_VAL);
#if 0
	/* For Sabre Lite*/
	/* UART1 TX : SD3_DAT6 */
	set_iomuxc(0x02A8,
			   1,
			   0x0000,
			   0,
			   0x0690,
			   UART_PAD_CTL_VAL);
	/* UART1 RX : SD3_DAT7 */
	set_iomuxc(0x02AC,
			   1,
			   0x0920,
			   3,
			   0x0694,
			   UART_PAD_CTL_VAL);
#endif
#elif defined(TARGET_USE_UART2)
	/* For FreeScale Sabre */
	/* For Sabre Lite*/
	/* UART2 TX : EIM_D26*/
	set_iomuxc(0x00BC,
			   4,
			   0x0000,
			   0,
			   0x03D0,
			   UART_PAD_CTL_VAL);
	/* UART2 RX : EIM_D27 */
	set_iomuxc(0x00C0,
			   4,
			   0x0928,
			   1,
			   0x03D4,
			   UART_PAD_CTL_VAL);
#elif defined(TARGET_USE_UART3)
	/* UART3 TX : EIM_D24 */
	set_iomuxc(0x00B4,
			   2,
			   0x0000,
			   0,
			   0x03C8,
			   UART_PAD_CTL_VAL);
	/* UART3 RX : EIM_D25 */
	set_iomuxc(0x00B8,
			   2,
			   0x0930,
			   1,
			   0x03CC,
			   UART_PAD_CTL_VAL);
#endif /* TARGET_USE_UART1 */
}

#ifdef TOPPERS_SAFEG_SECURE
uint32_t
target_get_nt_start_address(void) {
	uint32_t addr;
	uint32_t gpr[] = {SRC_GPR1, SRC_GPR3, SRC_GPR5, SRC_GPR7};

	if ((addr = sil_rew_mem((uint32_t*)gpr[x_prc_index()])) == 0)
	  return -1;

	return addr;
}

void
target_prepare_reboot(void){

}

/*
 *  Reboot Non-Trust Linux
 */
void
target_do_reboot(void){

}

#endif /* TOPPERS_SAFEG_SECURE */
