/*
 *  TOPPERS/FMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Flexible MultiProcessor Kernel
 *
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 *  Copyright (C) 2006-2012 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 *
 *  嵭Ԥϡʲ(1)(4)ξ˸¤ꡤܥեȥ
 *  ܥեȥѤΤޤࡥʲƱˤѡʣ
 *  ѡۡʰʲѤȸƤ֡ˤ뤳Ȥ̵ǵ롥
 *  (1) ܥեȥ򥽡ɤηѤˤϡ嵭
 *      ɽѾ浪Ӳ̵ݾڵ꤬Τޤޤηǥ
 *      ˴ޤޤƤ뤳ȡ
 *  (2) ܥեȥ򡤥饤֥ʤɡ¾Υեȥȯ˻
 *      ѤǤǺۤˤϡۤȼɥȡ
 *      ԥޥ˥奢ʤɡˤˡ嵭ɽѾ浪Ӳ
 *      ̵ݾڵǺܤ뤳ȡ
 *  (3) ܥեȥ򡤵Ȥ߹ʤɡ¾Υեȥȯ˻
 *      ѤǤʤǺۤˤϡΤ줫ξ
 *      ȡ
 *    (a) ۤȼɥȡѼԥޥ˥奢ʤɡˤˡ嵭
 *        ɽѾ浪Ӳ̵ݾڵǺܤ뤳ȡ
 *    (b) ۤη֤̤ˡˤäơTOPPERSץȤ
 *        𤹤뤳ȡ
 *  (4) ܥեȥѤˤľŪޤϴŪ뤤ʤ»
 *      ⡤嵭ԤTOPPERSץȤդ뤳ȡ
 *      ޤܥեȥΥ桼ޤϥɥ桼Τʤ
 *      ͳ˴Ťᤫ⡤嵭ԤTOPPERSץȤ
 *      դ뤳ȡ
 *
 *  ܥեȥϡ̵ݾڤ󶡤ƤΤǤ롥嵭Ԥ
 *  TOPPERSץȤϡܥեȥ˴ؤơλŪ
 *  ФŬޤơʤݾڤԤʤޤܥեȥ
 *  ѤˤľŪޤϴŪʤ»˴ؤƤ⡤
 *  Ǥʤ
 *
 */
#ifndef TOPPERS_TARGET_CONFIG_H
#define TOPPERS_TARGET_CONFIG_H

#include "imx6.h"

#ifdef TOPPERS_NOSAFEG
#define OMIT_VBAR_SETUP
#endif /* TOPPERS_NOSAFEG */

/*
 *  Enable trace (use the ENABLE_TRACE flag at the app's Makefile)
 *
 */
#ifdef TOPPERS_ENABLE_TRACE
#include "logtrace/trace_config.h"
#endif /* TOPPERS_ENABLE_TRACE */

/*
 * Enable performance counter (global private timer)
 */
// #define TOPPERS_ENA_PERF

/*
 *  Use the ipi_handler bypass processing
 *
 *  Note: After one core (e.g. core0) changes the status of a task
 *  in another core (e.g., act_tsk(CORE1TASK)) usually an interrupt is
 *  sent to that core (e.g., core1) for dispatching. If we use the
 *  USE_IPI_DIS_HANDER_BYPASS macro, instead of going to an interrupt
 *  handler the dispatcher will be called automatically for reducing
 *  overhead.
 */
#define USE_IPI_DIS_HANDER_BYPASS

/*
 *  Spinlock implementation to use
 *      - NATIVE_SPN: no contention with other syscalls
 *      - EMULATE_SPN: contention with other syscalls
 */
#define TTYPE_SPN NATIVE_SPN
//#define TTYPE_SPN EMULATE_SPN

/*
 *  Maximum number of unsuccesful spins (Native Spinlock)
 */
#define TMAX_NATIVE_SPN   256

/*
 *  Default size of non-task stack for storing context
 */
#define DEFAULT_PRC1_ISTKSZ      (1024 * 8)   /* 8KB */
#define DEFAULT_PRC2_ISTKSZ      (1024 * 8)   /* 8KB */
#define DEFAULT_PRC3_ISTKSZ      (1024 * 8)   /* 8KB */
#define DEFAULT_PRC4_ISTKSZ      (1024 * 8)   /* 8KB */

/*
 *  Include chip configuration (MPCORE)
 */
#include "arm_gcc/mpcore/chip_config.h"

#ifndef TOPPERS_MACRO_ONLY

/*
 * Memory map initialization (called from the chip-dependent code)
 */
extern void target_mmu_init(void);

/*
 *  Target-dependent initialization
 */
extern void target_initialize(void);

/*
 *  Function to execute when the system exits
 */
extern void target_exit(void) NoReturn;

#endif /* TOPPERS_MACRO_ONLY */

#endif /* TOPPERS_TARGET_CONFIG_H */
